'''
Created on 15.04.2010

@author: gsu
'''

import pyodbc
import re
from nes_common import logger, rest_client
from nes_common.executor import Executor

log = None

class TableManager(object):

    def __init__(self, cursor):
        self.cursor = cursor
        self.workTables = {}

    def invertName(self, base, name):
        return base + '1' if name.endswith('2') else base + '2'

    def prepareWorkingCopy(self, tableBase, tableInUse):
        tableWorking = self.invertName(tableBase, tableInUse)
        log.info('cleaning %s' % tableWorking)
        self.cursor.execute('TRUNCATE TABLE %s' % (tableWorking,))
        log.info('copying %s to %s' % (tableInUse, tableWorking))
        self.cursor.execute("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.Columns where TABLE_NAME = '%s'" % tableWorking)
        columns = ''
        for columnName in self.cursor.fetchall():
            columns = columns + columnName[0] + ', '
        self.cursor.execute("select IDENT_SEED('%s')" % tableWorking)
        query = 'insert into %s (%s) select * from %s ; ' % (tableWorking, columns[:-2], tableInUse)
        if self.cursor.fetchone()[0]:
            qqq = 'SET IDENTITY_INSERT %s ON; ' % tableWorking
            query = qqq + query
            qqq = 'SET IDENTITY_INSERT %s OFF;' % (tableWorking)
            query = query + qqq
        self.cursor.execute(query)
        return tableWorking


    def getWorkingCopy(self, table):
        if self.workTables.get(table, None) == None:
            try:
                self.cursor.execute('sp_helptext @objname = %s' % (table,))
                row = self.cursor.fetchone()
                matches = re.search('%s[12]' % (table,), row[0])
                if matches != None:
                    result = self.prepareWorkingCopy(table, matches.group(0))
                    log.info('will work with %s' % (result,))
                    self.workTables[table] = result
            except Exception, e:
                log.info('%s is not view' % (table,))
                log.exception(e)
                self.workTables[table] = table
        return self.workTables[table]
    
    def finalize(self):
        log.info('all tables are OK. altering views')
        for view, table in self.workTables.items():
            if view != table:
                self.cursor.execute('alter view %s as select * from %s' % (view,table))
        log.info('views are altered now')
        return True

class Context(object):
    cursor = None
    def __init__(self, cursor, config):
        self.cursor = cursor
        self.tableManager = TableManager(self.cursor)
        self.emptyTables = {}
        try:
            self.executeMany = config.getboolean('mssql', 'executeMany')
        except:
            self.executeMany = True

    def tableName(self, table):
        return self.tableManager.getWorkingCopy(table)

    def finalize(self):
        return self.tableManager.finalize()

class Exec(Executor):
    def __init__(self, driver, server, user, password, database, shopCode, config, loadDictStatusSender=None):
        global log
        try:
            commandCommitCount = config.getint('mssql', 'commandCommitCount')
        except:
            commandCommitCount = 5000
        log = logger.Logger().getLogger(shopCode)
        connection = pyodbc.connect('driver=%s;server=%s;uid=%s;pwd=%s;database=%s' % (driver, server, user, password, database))
        context = Context(connection.cursor(), config)
        loadDictStatusSender.sendDictState('Start load dict to db', 'Загрузка справочника в БД касы', rest_client.STATUS_DURING)
        Executor.__init__(self, shopCode, connection, context, commandCommitCount)

