# -*- coding: UTF-8 -*-

'''
Created on 15.04.2010

@author: gsu
'''

import MySQLdb
import re, shutil, os, sys
from nes_common import logger, rest_client
from nes_common.sql_operation import _Insert
from nes_common.executor import Executor, DataIntegrityError

log = None

class Context(object):
    cursor = None
    def __init__(self, cursor, config, shopCode, loadDictStatusSender = None):
        self.cursor = cursor
        self.emptyTables = {}
        self.shopCode = shopCode
        self.loadDictStatusSender = loadDictStatusSender
        self.tables = []
        try:
            self.executeMany = config.getboolean('mysql', 'executeMany')
        except:
            self.executeMany = True

    def tableName(self, table):
        if not self.tables.__contains__(table) and self.loadDictStatusSender:
            self.tables.append(table)
            log.info('table = %s' % table)
            self.loadDictStatusSender.sendDictState('Start load dict to db',
                    'Загрузка справочника в БД кассового сервера', rest_client.STATUS_DURING, tableName=table)
        return table

    def finalize(self):
        return True

class Exec(Executor):
    migrateVersions = {}
    def __init__(self, host, port, user, password, database, shopCode, config, loadDictStatusSender = None):
        global log
        self.database = database
        self.clearCache = False
        try:
            commandCommitCount = config.getint('mysql', 'commandCommitCount')
        except:
            commandCommitCount = 5000
        log = logger.Logger().getLogger(shopCode)
        if not password or password == '':
            connection = MySQLdb.connect(host = host, port = port, user=user, db=database)
            self.connectionAll = MySQLdb.connect(host = host, port = port, user=user, db=database.replace('dictionaries', "artixcs"))
        else:
            connection = MySQLdb.connect(host = host, port = port, user=user, passwd=password, db=database)
            self.connectionAll = MySQLdb.connect(host = host, port = port, user=user, passwd=password, db=database.replace('dictionaries', "artixcs"))
        self.connectionAll.autocommit(True)
        connection.set_character_set('utf8')
        connection.ping(True)
        context = Context(connection.cursor(), config, shopCode, loadDictStatusSender)
        Executor.__init__(self, shopCode, connection, context, commandCommitCount)

    def execute(self, sql):
        if self.clearCache and isinstance(sql, _Insert):
            self.log.debug('Old cache: %s' % sql.cacheOddColumns)
            sql.cacheOddColumns.clear()
            self.log.debug('New cache: %s' % sql.cacheOddColumns)
            sql.cacheOwnerStoreIdPresented.clear()
            self.clearCache = False
        Executor.execute(self, sql)

    def _execFinalCheckRemoveRecords(self):
        if self.context.emptyTables:
            for tableName in self.context.emptyTables:
                try:
                    self.context.cursor.callproc("finalCheckRemovedRecords", [tableName])
                except Exception, e:
                    self.log.warning("Can't execute stored function: %s", e.message)

    def startUpload(self, messageId):
        Executor.startUpload(self, messageId)
        try:
            cur = self.connectionAll.cursor()
            cur.execute("SELECT version FROM migrate_version WHERE repository_id LIKE 'artixcs%';")
            row = cur.fetchone()
            currentMigrateVersion = Exec.migrateVersions.get(self.database)
            dbVersion = row[0] if row != None else currentMigrateVersion
            if currentMigrateVersion != dbVersion and currentMigrateVersion != None:
                self.log.info('migrateVersion is changed from %s to %s. Clear cache with tables columns' % (currentMigrateVersion, dbVersion))
                self.clearCache = True
            Exec.migrateVersions[self.database] = dbVersion
        except Exception, e:
            self.log.warning("Can't get migrate version. %s" % e)
        finally:
            cur.close()
            self.connectionAll.close()

    def flushTables(self):
        # Для КС таблицы флушить не нужно(т.к. никаких хитрых операций с файлами БД на ФС не производится в режиме КС)
        pass
